/* packet-ospf.c
 * Routines for OSPF packet disassembly
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@zing.org>
 * Copyright 1998 Gerald Combs
 * 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
#include "config.h"

#include <gtk/gtk.h>

#include <stdio.h>

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif

#include "ethereal.h"
#include "packet.h"
#include "packet-ospf.h"


void 
dissect_ospf(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
    e_ospfhdr *ospfh;

    GtkWidget *ospf_tree = NULL, *ti, *ti2; 
    GtkWidget *ospf_header_tree;
    char auth_data[9]="";

    ospfh = (e_ospfhdr *) &pd[offset];

    if (fd->win_info[0]) {
        strcpy(fd->win_info[3], "OSPF");
        sprintf(fd->win_info[4], "OSPF Packet Type: %d", ospfh->packet_type);
    }  

    if (tree) {
	ti = add_item_to_tree(GTK_WIDGET(tree), offset, ntohs(ospfh->length), "Open Shortest Path First"); 
	ospf_tree = gtk_tree_new(); 
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), ospf_tree);

	ti2 = add_item_to_tree(GTK_WIDGET(ospf_tree), offset, 24, "OSPF Header"); 
	ospf_header_tree = gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti2), ospf_header_tree);

        add_item_to_tree(ospf_header_tree, offset, 1, "OSPF Version: %d", ospfh->version);  
	switch(ospfh->packet_type) {
	    case OSPF_HELLO:
            	add_item_to_tree(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d (Hello Packet)", ospfh->packet_type);  
		break;
	    case OSPF_DB_DESC:
            	add_item_to_tree(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d (Database Descr.)", ospfh->packet_type);  
		break;
	    case OSPF_LS_REQ:
            	add_item_to_tree(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d (LS Request)", ospfh->packet_type);  
		break;
	    case OSPF_LS_UPD:
            	add_item_to_tree(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d (LS Update)", ospfh->packet_type);  
		break;
	    case OSPF_LS_ACK:
            	add_item_to_tree(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d (LS Acknowlege)", ospfh->packet_type);  
		break;
	    default:
	        add_item_to_tree(ospf_header_tree, offset + 1 , 1, "OSPF Packet Type: %d", ospfh->packet_type);
	}
	add_item_to_tree(ospf_header_tree, offset + 2 , 2, "Packet Legth: %d", ntohs(ospfh->length));
	add_item_to_tree(ospf_header_tree, offset + 4 , 4, "Source OSPF Router ID: %s", ip_to_str((guint8 *) &(ospfh->routerid)));
	if (!(ospfh->area)) {
	   add_item_to_tree(ospf_header_tree, offset + 8 , 4, "Area ID: Backbone");
	} else {
	   add_item_to_tree(ospf_header_tree, offset + 8 , 4, "Area ID: %s", ip_to_str((guint8 *) &(ospfh->area)));
	}
	add_item_to_tree(ospf_header_tree, offset + 12 , 2, "Packet Checksum");
	switch( ntohs(ospfh->auth_type) ) {
	    case OSPF_AUTH_NONE:
	         add_item_to_tree(ospf_header_tree, offset + 14 , 2, "Auth Type: none");
	         add_item_to_tree(ospf_header_tree, offset + 16 , 8, "Auth Data (none)");
		 break;
	    case OSPF_AUTH_SIMPLE:
	         add_item_to_tree(ospf_header_tree, offset + 14 , 2, "Auth Type: simple");
                 strncpy(auth_data, &(ospfh->auth_data), 8);
	         add_item_to_tree(ospf_header_tree, offset + 16 , 8, "Auth Data: %s", auth_data);
		 break;
	    case OSPF_AUTH_CRYPT:
	         add_item_to_tree(ospf_header_tree, offset + 14 , 2, "Auth Type: crypt");
	         add_item_to_tree(ospf_header_tree, offset + 16 , 8, "Auth Data (crypt)");
		 break;
            default:
	         add_item_to_tree(ospf_header_tree, offset + 14 , 2, "Auth Type (unknown)");
	         add_item_to_tree(ospf_header_tree, offset + 16 , 8, "Auth Data (unknown)");
	}
    }

    /*  Skip over header */
    offset += 24;
    switch(ospfh->packet_type){
	case OSPF_HELLO:
	    dissect_ospf_hello(pd, offset, fd, (GtkTree *) ospf_tree);
	    break;
	case OSPF_DB_DESC:
	    break;
	case OSPF_LS_REQ:
	    break;
	case OSPF_LS_UPD:
	    break;
	case OSPF_LS_ACK:
	    break;
	default:
            dissect_raw(pd, offset, fd, tree); 
    }
}

void
dissect_ospf_hello(const u_char *pd, int offset, frame_data *fd, GtkTree *tree) {
    e_ospf_hello *ospfhello;

    GtkWidget *ospf_hello_tree, *ti; 
    
    char auth_data[9]="";
    if (tree) {
	ti = add_item_to_tree(GTK_WIDGET(tree), offset, (fd->cap_len -
        offset) , "OSPF Hello Packet"); 
	ospf_hello_tree = gtk_tree_new(); 
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(ti), ospf_hello_tree);
    }
}
